using System.Net.Mime;
using FitConnect;
using FitConnect.Models;
using FitConnect.Models.v1.Api;
using FluentAssertions;
using Microsoft.Extensions.Logging;

namespace E2ETest;

[Order(10)]
public class RejectSubmissionTest : EndToEndTestBase {
    private SentSubmission _submission = null!;

    [Order(5)]
    [Test]
    public void CleanServer() {
        var submissions = Subscriber.GetAvailableSubmissionsAsync(Settings.DestinationId).Result;
        foreach (var submission in submissions)
            try {
                Subscriber.RejectSubmissionAsync(submission,
                        new Problems(Problems.ProblemTypeEnum.MissingData, "Server cleanup"))
                    .Wait();
            }
            catch (Exception e) {
                Logger.LogError(e, "Error rejecting submission {SubmissionId}", submission);
            }

        Subscriber.GetAvailableSubmissionsAsync(Settings.DestinationId).Result.Count().Should()
            .Be(0);
    }


    [Order(10)]
    [Test]
    public void SendingSubmission() {
        var outgoing = SendableSubmission.Builder()
            .SetDestination(Settings.DestinationId)
            .SetServiceType(Settings.ServiceIdentifier, "Straight forward test")
            .SetXmlData(@"<user><name>John Doe</name></user>")
            .AddAttachments(Attachment.FromPath("Test.pdf","application/pdf", description: "A simple PDF", fileName: "Test.pdf"))
            .Build();

        _submission = Sender.SendAsync(outgoing).Result;
    }

    [Test]
    [Order(20)]
    public void Sender_GetSubmissionState() {
        // Act
        var status = Sender.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void Reject_Submission() {
        var receivedSubmission =
            Subscriber.RequestSubmissionAsync(_submission.SubmissionId).Result;

        var data = receivedSubmission.GetData();
        Logger.LogInformation("Data {Data}", data);

        receivedSubmission.GetDataMimeType().Should()
            .Be(MediaTypeNames.Application.Xml);

        data.Should().Be(@"<user><name>John Doe</name></user>");

        var attachments = receivedSubmission.GetAttachmentsAsync().Result;
        attachments.First().Filename.Should().Be("Test.pdf");

        receivedSubmission.RejectSubmissionAsync(
            new Problems(Problems.ProblemTypeEnum.UnsupportedReplyChannel, ProblemDetail)
        ).Wait();
    }

    private const string ProblemDetail = "Reject submission test";

    [Test]
    [Order(50)]
    public void Sender_GetSubmissionState_AfterRejecting() {
        // Act
        var status = Sender.GetEventLogAsync(_submission.CaseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));

        var rejection = status.First(s => s.EventType == EventType.Reject);

        rejection.Events?.RejectSubmissionEvent?.Problems.Should().NotBeNull();
        rejection.Events?.RejectSubmissionEvent?.Problems?.ForEach(p =>
            Logger.LogWarning(p.Detail));
    }

    [Test]
    [Order(60)]
    public void Sender_GetStatus() {
        var (status, problems) =
            Sender.GetStatusForSubmissionAsync(_submission).Result;

        status.Should().Be(EventType.Reject, "The submission should have been rejected");
        problems!.Any(p => p.Detail == ProblemDetail).Should().BeTrue();
    }
}
